#ifndef STARS_irdb_Interface_h
#define STARS_irdb_Interface_h


#include "interfaces/abstract/STARSInterface.h"
#include "interfaces/irdb/STARSSegment.h"
#include <elfio/elfio.hpp>
#include <elfio/elfio_dump.hpp>

#include <libIRDB-core.hpp>
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <pqxx/pqxx>

class STARS_IRDB_Interface_t: public STARS_Interface_t
{
public:

// Constructors
	STARS_IRDB_Interface_t(FileIR_t* p_firp, libIRDB::pqxxDB_t &p_pqxx_interface) : 
		firp(p_firp), pqxx_interface(p_pqxx_interface) 
	{ 
		InitSegments();
	}
		
// Segment accessors

	virtual STARS_Segment_t *getseg(const STARS_ea_t &addr) { /* iterate through sections */ assert(0); return NULL; }
	virtual STARS_Segment_t *getnseg(const int &index) { return segments[index]; }
	virtual std::size_t get_segm_qty() { return segments.size(); }
	virtual STARS_Segment_t *get_next_seg(const STARS_ea_t &addr) { /* iterate-solution */assert(0); return NULL; }

// Function accessors

    	// find out how many functions there are
	virtual std::size_t get_func_qty() {  return firp->GetFunctions().size(); }  

    	// get the index-th function
	virtual STARS_Function_t *getn_func(int index) 
	{ /* easy */assert(0); return 0; }; 

	// get the function that has the instruction 'instruction_id' as the entry point.
	virtual STARS_Function_t *get_func(STARS_ea_t instruction_id) 
	{ / assert(0); return NULL; }; 

	virtual void get_func_name(const STARS_ea_t &ea, char* name, const std::size_t &len) 
	{ /* easy */assert(0); }; 

// Instruction creation.
	virtual STARS_Instruction_t *CreateInst(STARS_InstructionID_t InstID) 
	{ /* see IDA version not hard. */ assert(0); return NULL; }; 

// File methods.
	virtual FILE *STARS_fopen(const char *file, const char *mode) 
	{ return fopen(file,mode); }
	virtual int STARS_fclose(FILE *fp)
	{ return fclose(fp); }
	virtual int STARS_msg(const char *format, ...)
	{ va_list ap; va_start(ap,format); return vfprintf(stderr, format, ap); }
	virtual int STARS_fprintf(FILE *fp, const char *format, ...)
	{ va_list ap; va_start(ap,format); return vfprintf(fp, format, ap); }
	virtual int STARS_fscanf(FILE *fp, const char *format, ...)
	{ va_list ap; va_start(ap,format); return vfscanf(fp, format, ap); }
	virtual long STARS_ftell(FILE *fp)
	{ return ftell(fp); }
	virtual char *STARS_fgets(char *buffer, int buflen, FILE *fp)
	{ return fgets(buffer,buflen,fp); }
	virtual int STARS_fseek(FILE *fp, long offset, int whence)
	{ return fseek(fp,offset,whence); }
	virtual int STARS_fgetc(FILE *fp)
	{ return fgetc(fp); }

// String methods
	virtual char *STARS_strncat(char *dst, const char *src, std::size_t dstsize) 
	{ return strncat(dst,src,dstsize); }
	virtual char *STARS_strncpy(char *dst, const char *src, std::size_t dstsize) 
	{ return strncpy(dst,src,dstsize); }
	virtual int STARS_snprintf(char *buffer, std::size_t n, const char *format, ...)
	{ va_list ap; va_start(ap,format); return vsnprintf(buffer,n, format, ap); }

// Disasm methods.
	virtual STARS_ssize_t STARS_tag_remove(const char *instr, char *buf, std::size_t bufsize) 
	{ 
		/* remove comments, etc. from disassembly.*/  
		// do strcpy from instr to buf 
		if(instr != buf)
			strcpy(buf,instr, bufsize);
		return 0; // success
	}
	virtual bool STARS_generate_disasm_line(STARS_ea_t addr, char *buf, std::size_t bufsize, int flags = 0) 
	{ /* call insn->getDisassembly() on instruction with id=='addr' */assert(0); return 0; }

// Miscellaneous methods.
	virtual bool InstHasNoCodeXrefs(STARS_InstructionID_t InstID) const
	{ assert(0); return 0; } // no impl'd, shouldn't be called.
	
	virtual bool IsInstJumpTarget(STARS_InstructionID_t InstID) const
	{ 
		// return true if 
		//	jumped to indirectly (jmp instruction)
		//	called indirectly (call instruction)
		// 	jumped to directly (jmp or jcc instruction)
		//	called directly (call instruction)

	
		// 	jumped to via a return doesn't count.
		// be careful and think about this as you impl it.

		// aka -- not (is only-fallthroughed to)
		assert(0); return 0; 
	}

// Find call target; first one, if any, for indirect call
	virtual STARS_InstructionID_t FindFirstCallTarget(STARS_InstructionID_t CallInstID) const 
	{ 
		// direct call: if no target -- return STARS_BADADDR
		// direct call: if target, return target 
		// indirect call: if any targets exist return one.
		// indirect call: if no known targets, return  STARS_BADADDR ?
		assert(0); return 0; 
	}

	// Miscellaneous IDA-only methods.
	virtual void AuditTailChunkOwnership(void)
	{ assert(0); return; /* shouldn't be called  */ }
    	virtual void AuditCodeTargets(void)
	{  assert(0); return; /* shouldn't be called.   */ }

	// Patch IDA Pro database.
	virtual bool STARS_patch_byte(STARS_ea_t InstAddr, uint32_t ByteValue) 
	{ assert(0); return 0; /* shouldn't be called */}

	
private:

	void InitSegments()
	{
		int elfoid=firp->GetFile()->GetELFOID();
		pqxx::largeobject lo(elfoid);
		lo.to_file(pqxx_interface.GetTransaction(),"readeh_tmp_file.exe");
//		elfiop.reset(new ELFIO::elfio);
		elfiop=new ELFIO::elfio;
		elfiop->load("readeh_tmp_file.exe");

		ELFIO::dump::header(cout,*elfiop);
		ELFIO::dump::section_headers(cout,*elfiop);

		segments.resize(elfiop->sections.size());
		for(int i=0;i<elfiop->sections.size(); i++)
		{
			segments[i]=new STARS_IRDB_Segment_t(elfiop->sections[i],i);
		}

	}

	vector<STARS_Segment_t*> segments;
//	unique_ptr<ELFIO::elfio> elfiop;
	ELFIO::elfio* elfiop;

	

	libIRDB::FileIR_t *firp;
	libIRDB::pqxxDB_t &pqxx_interface;
};


#endif


